from django.apps import AppConfig


class DjangoClickhouseLoggerConfig(AppConfig):
    name = 'django_logger_clickhouse'
    verbose_name = 'ClickHouse Django Logger'

    def ready(self):
        from django.conf import settings

        if self.name in settings.INSTALLED_APPS:
            from .utils import ensure_clickhouse_db_and_table_exist
            from .config import check_settings

            check_settings()
            ensure_clickhouse_db_and_table_exist()



import io
import logging
import traceback
from uuid import uuid4
import datetime
import json

from .tasks import record_in_clickhouse


class ClickHouseLogger():
    def format_exception(ei) -> str:
        sio = io.StringIO()
        tb = ei[2]
        traceback.print_exception(ei[0], ei[1], tb, None, sio)
        s = sio.getvalue()
        sio.close()
        if s[-1:] == "\n":
            s = s[:-1]
        return s

    def logger(record: logging.LogRecord = "") -> None:
        data = {}
        data["uuid"] = str(uuid4())
        data["created_dt"] = datetime.datetime.now().strftime(
            "%Y-%m-%d %H:%M:%S.%f")
        data["pathname"] = str(getattr(record, "pathname", ""))
        data["funcName"] = str(getattr(record, "funcName", ""))
        data["lineno"] = getattr(record, "lineno", 0)
        data["message"] = record.getMessage()
        data["exc_text"] = str(getattr(record, "exc_text", ""))
        data["created"] = getattr(record, "lineno", 0)
        data["filename"] = str(getattr(record, "filename", ""))
        data["levelname"] = str(getattr(record, "levelname", ""))
        data["levelno"] = str(getattr(record, "levelno", ""))
        data["module"] = str(getattr(record, "module", ""))
        data["msecs"] = getattr(record, "msecs", 0)
        data["msg"] = str(getattr(record, "msg", ""))
        data["name"] = str(getattr(record, "name", ""))
        data["process"] = str(getattr(record, "process", ""))
        data["processName"] = str(getattr(record, "processName", ""))
        data["relativeCreated"] = str(getattr(record, "relativeCreated", ""))
        data["stack_info"] = str(getattr(record, "stack_info", ""))
        data["thread"] = str(getattr(record, "thread", ""))
        data["threadName"] = str(getattr(record, "threadName", ""))
        if data["msg"] == '%s: %s':
            data["msg"] = data["exc_text"]
        return json.dumps(data)


class LoggerHandler(logging.StreamHandler):

    def emit(self, record) -> None:
        if isinstance(record, logging.LogRecord):
            try:
                if record.exc_info:  # Check if exception info exists
                    record.exc_text = "".join(traceback.format_exception(
                        *record.exc_info))  # Format exception info
                data = ClickHouseLogger.logger(record)
                record_in_clickhouse.delay(data, 'logs')
            except Exception as e:
                logging.exception(f"django-clickhouse-logger [ERROR] {e}")



import logging
import requests
import json
from django.conf import settings

from .config import app


logger = logging.getLogger(__name__)

use_logger_queue = getattr(settings, 'USE_LOGGER_QUEUE_NAME', None)
task_decorator = app.task(queue=use_logger_queue) if use_logger_queue else app.task

@task_decorator
def record_in_clickhouse(data, *args):
    """ Производит запись в clickhouse """

    data = json.loads(data)
    data["server_name"] = getattr(
        settings, 'DJANGO_CLICKHOUSE_SERVER_NAME', 'development')
    clickhouse_host = settings.DJANGO_CLICKHOUSE_LOGGER_HOST
    clickhouse_port = settings.DJANGO_CLICKHOUSE_LOGGER_PORT
    clickhouse_user = settings.DJANGO_CLICKHOUSE_LOGGER_USER
    clickhouse_password = settings.DJANGO_CLICKHOUSE_LOGGER_PASSWORD

    query = "INSERT INTO clickhouse.logs FORMAT JSONEachRow"
    url = f"http://{clickhouse_host}:{clickhouse_port}/"
    auth = (clickhouse_user, clickhouse_password)
    try:
        response = requests.post(url, params={"query": query}, data=json.dumps(data), auth=auth)
    except Exception as e:
        logger.error(f'Error connection clickhouse {e}')
        print('Error connection in clickhouse', e)
        return

    if response.status_code != 200:
        logger.error(f'Error clickhouse status {response.text}')
        print(f"Error clickhouse status: {response.text}")



import requests

from django.conf import settings

from .config import check_settings


def ensure_clickhouse_db_and_table_exist():
    check_settings()

    clickhouse_host = settings.DJANGO_CLICKHOUSE_LOGGER_HOST
    clickhouse_port = settings.DJANGO_CLICKHOUSE_LOGGER_PORT
    clickhouse_user = settings.DJANGO_CLICKHOUSE_LOGGER_USER
    clickhouse_password = settings.DJANGO_CLICKHOUSE_LOGGER_PASSWORD

    # Строим URL для ClickHouse
    url = f"http://{clickhouse_host}:{clickhouse_port}/"

    # SQL запросы для создания базы данных и таблицы
    create_db_query = "CREATE DATABASE IF NOT EXISTS clickhouse;"
    create_table_query = """
    CREATE TABLE IF NOT EXISTS clickhouse.logs
    (
      uuid String,
      created_dt DateTime64(3),
      pathname Nullable(String),
      funcName Nullable(String),
      lineno Nullable(Int32),
      message Nullable(String),
      exc_text Nullable(String),
      created Nullable(Float64),
      filename Nullable(String),
      levelname Nullable(String),
      levelno Nullable(String),
      module Nullable(String),
      msecs Nullable(Float64),
      msg Nullable(String),
      name Nullable(String),
      process Nullable(String),
      processName Nullable(String),
      relativeCreated Nullable(String),
      stack_info Nullable(String),
      thread Nullable(String),
      threadName Nullable(String),
      server_name Nullable(String)
    )
    ENGINE = MergeTree()
    PARTITION BY toDate(created_dt)
    ORDER BY (created_dt)
    SETTINGS min_bytes_for_wide_part = 0;
    """

    # Выполнение запросов
    try:
        response = requests.post(url, params={"query": create_db_query}, auth=(clickhouse_user, clickhouse_password))
        response.raise_for_status()
        response = requests.post(url, params={"query": create_table_query}, auth=(clickhouse_user, clickhouse_password))
        response.raise_for_status()
    except requests.exceptions.RequestException as e:
        raise Exception(f"django_logger_clickhouse error: Failed to ensure ClickHouse DB and table exist: {e}")



from setuptools import setup, find_packages

# Читаем файл README.md для длинного описания
with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="django_logger_clickhouse",
    version="0.1.2",
    author="Алексей",
    author_email="vipzenit666@yandex.ru",
    description="Логгер Django для ClickHouse с Celery.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    # url="https://github.com/single-service/clickhouse_django_logger",
    # project_urls={
    #     "Homepage": "https://github.com/single-service/clickhouse_django_logger",
    #     "Issues": "https://github.com/single-service/clickhouse_django_logger/issues",
    # },
    classifiers=[
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    license="MIT",
    packages=find_packages(exclude=["tests*"]),
    python_requires=">=3.6",
    install_requires=[
        "Django>=2.2",
        "celery>=5.0",
        "requests>=2.0",
    ],
    extras_require={
        "dev": ["pytest", "mypy", "flake8"],
        "docs": ["sphinx"],
    },
    include_package_data=True,
    zip_safe=False,
)



from __future__ import absolute_import, unicode_literals
import os

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

from celery import Celery

os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'config.settings')
app = Celery('config')
app.config_from_object('django.conf:settings', namespace='CELERY')

REQUIRED_SETTINGS = [
    'DJANGO_CLICKHOUSE_LOGGER_HOST',
    'DJANGO_CLICKHOUSE_LOGGER_PORT',
    'DJANGO_CLICKHOUSE_LOGGER_USER',
    'DJANGO_CLICKHOUSE_LOGGER_PASSWORD'
]


def check_settings():
    if 'django_logger_clickhouse' in settings.INSTALLED_APPS:
        for setting_name in REQUIRED_SETTINGS:
            if not hasattr(settings, setting_name):
                raise ImproperlyConfigured(
                    f'django_logger_clickhouseerror: Missing required setting: {setting_name}')
